@@ -10,8 +10,8 @@ except ImportError:
 
 from trytond.i18n import gettext
 from trytond.model import (
-    DeactivableMixin, ModelSQL, ModelView, MultiValueMixin, ValueMixin, fields,
-    sequence_ordered)
+    DeactivableMixin, Index, ModelSQL, ModelView, MultiValueMixin, ValueMixin,
+    fields, sequence_ordered)
 from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
@@ -38,6 +38,31 @@ _PHONE_TYPES = {
     }
 
 
+class _ContactMechanismMixin:
+    __slots__ = ()
+
+    def contact_mechanism_get(self, types=None, usage=None):
+        """
+        Try to find a contact mechanism for the given types and usage, if no
+        usage matches the first mechanism of the given types is returned.
+        """
+        default_mechanism = None
+        if types:
+            if isinstance(types, str):
+                types = {types}
+            mechanisms = [m for m in self.contact_mechanisms
+                if m.type in types]
+        else:
+            mechanisms = self.contact_mechanisms
+        if mechanisms:
+            default_mechanism = mechanisms[0]
+            if usage:
+                for mechanism in mechanisms:
+                    if getattr(mechanism, usage):
+                        return mechanism
+        return default_mechanism
+
+
 class ContactMechanism(
         DeactivableMixin, sequence_ordered(), ModelSQL, ModelView,
         MultiValueMixin):
@@ -46,7 +71,8 @@ class ContactMechanism(
     _rec_name = 'value'
 
     type = fields.Selection(_TYPES, "Type", required=True, sort=False)
-    value = fields.Char("Value", select=True,
+    value = fields.Char(
+        "Value",
         # Add all function fields to ensure to always fill them via on_change
         depends={
             'email', 'website', 'skype', 'sip', 'other_value',
@@ -55,7 +81,12 @@ class ContactMechanism(
     name = fields.Char("Name")
     comment = fields.Text("Comment")
     party = fields.Many2One(
-        'party.party', "Party", required=True, ondelete='CASCADE', select=True)
+        'party.party', "Party", required=True, ondelete='CASCADE')
+    address = fields.Many2One(
+        'party.address', "Address", ondelete='CASCADE',
+        domain=[
+            ('party', '=', Eval('party', -1)),
+            ])
     language = fields.MultiValue(
         fields.Many2One('ir.lang', "Language",
             help="Used to translate communication made "
@@ -95,7 +126,12 @@ class ContactMechanism(
 
     @classmethod
     def __setup__(cls):
+        cls.value.search_unaccented = False
+        cls.value_compact.search_unaccented = False
         super(ContactMechanism, cls).__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.add(
+            Index(t, (Index.Unaccent(t.value_compact), Index.Similarity())))
         cls._order.insert(0, ('party.distance', 'ASC NULLS LAST'))
         cls._order.insert(1, ('party', 'ASC'))
 
@@ -107,6 +143,11 @@ class ContactMechanism(
     def default_party(cls):
         return Transaction().context.get('related_party')
 
+    @fields.depends('address', '_parent_address.party')
+    def on_change_address(self):
+        if self.address:
+            self.party = self.address.party
+
     @classmethod
     def get_value(cls, mechanisms, names):
         return dict((name, dict((m.id, m.value) for m in mechanisms))
@@ -295,5 +336,5 @@ class ContactMechanismLanguage(ModelSQL, ValueMixin):
     __name__ = 'party.contact_mechanism.language'
     contact_mechanism = fields.Many2One(
         'party.contact_mechanism', "Contact Mechanism",
-        ondelete='CASCADE', select=True)
+        ondelete='CASCADE')
     language = fields.Many2One('ir.lang', "Language")
