@@ -3,28 +3,30 @@
 import stdnum.exceptions
 from sql import Column, Literal, Null
 from sql.aggregate import Min
-from sql.functions import CharLength, Position, Substring
+from sql.functions import CharLength
 from stdnum import get_cc_module
 
 from trytond import backend
 from trytond.i18n import gettext
 from trytond.model import (
-    DeactivableMixin, ModelSQL, ModelView, MultiValueMixin, Unique, ValueMixin,
-    fields, sequence_ordered)
+    DeactivableMixin, Index, ModelSQL, ModelView, MultiValueMixin, Unique,
+    ValueMixin, fields, sequence_ordered)
 from trytond.model.exceptions import AccessError
 from trytond.pool import Pool
 from trytond.pyson import Bool, Eval
-from trytond.tools import lstrip_wildcard
+from trytond.tools import is_full_text, lstrip_wildcard
 from trytond.tools.multivalue import migrate_property
 from trytond.transaction import Transaction
 from trytond.wizard import Button, StateTransition, StateView, Wizard
 
-from .contact_mechanism import _PHONE_TYPES
+from .contact_mechanism import _PHONE_TYPES, _ContactMechanismMixin
 from .exceptions import (
     EraseError, InvalidIdentifierCode, SimilarityWarning, VIESUnavailable)
 
 
-class Party(DeactivableMixin, ModelSQL, ModelView, MultiValueMixin):
+class Party(
+        DeactivableMixin, _ContactMechanismMixin, ModelSQL, ModelView,
+        MultiValueMixin):
     "Party"
     __name__ = 'party.party'
 
@@ -33,9 +35,10 @@ class Party(DeactivableMixin, ModelSQL, ModelView, MultiValueMixin):
         }
 
     name = fields.Char(
-        "Name", select=True,
+        "Name", strip=False,
         help="The main identifier of the party.")
-    code = fields.Char('Code', required=True, select=True,
+    code = fields.Char(
+        "Code", required=True,
         states={
             'readonly': Eval('code_readonly', True),
             },
@@ -87,11 +90,16 @@ class Party(DeactivableMixin, ModelSQL, ModelView, MultiValueMixin):
 
     @classmethod
     def __setup__(cls):
+        cls.code.search_unaccented = False
         super(Party, cls).__setup__()
         t = cls.__table__()
         cls._sql_constraints = [
             ('code_uniq', Unique(t, t.code), 'party.msg_party_code_unique')
             ]
+        cls._sql_indexes.update({
+                Index(t, (t.code, Index.Equality())),
+                Index(t, (t.code, Index.Similarity())),
+                })
         cls._order.insert(0, ('distance', 'ASC NULLS LAST'))
         cls._order.insert(1, ('name', 'ASC'))
         cls.active.states.update({
@@ -151,10 +159,11 @@ class Party(DeactivableMixin, ModelSQL, ModelView, MultiValueMixin):
     @classmethod
     def search_tax_identifier(cls, name, clause):
         _, operator, value = clause
+        nested = clause[0][len(name) + 1:]
         types = cls.tax_identifier_types()
         domain = [
             ('identifiers', 'where', [
-                    ('code', operator, value),
+                    (nested or 'rec_name', operator, value),
                     ('type', 'in', types),
                     ]),
             ]
@@ -295,18 +304,19 @@ class Party(DeactivableMixin, ModelSQL, ModelView, MultiValueMixin):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        if clause[1].startswith('!') or clause[1].startswith('not '):
+        _, operator, operand, *extra = clause
+        if operator.startswith('!') or operator.startswith('not '):
             bool_op = 'AND'
         else:
             bool_op = 'OR'
-        code_value = clause[2]
-        if clause[1].endswith('like'):
-            code_value = lstrip_wildcard(clause[2])
+        code_value = operand
+        if operator.endswith('like') and is_full_text(operand):
+            code_value = lstrip_wildcard(operand)
         return [bool_op,
-            ('code', clause[1], code_value) + tuple(clause[3:]),
-            ('identifiers.code', clause[1], code_value) + tuple(clause[3:]),
-            ('name',) + tuple(clause[1:]),
-            ('contact_mechanisms.rec_name',) + tuple(clause[1:]),
+            ('code', operator, code_value, *extra),
+            ('identifiers.code', operator, code_value, *extra),
+            ('name', operator, operand, *extra),
+            ('contact_mechanisms.rec_name', operator, operand, *extra),
             ]
 
     def address_get(self, type=None):
@@ -323,33 +333,12 @@ class Party(DeactivableMixin, ModelSQL, ModelView, MultiValueMixin):
                         return address
         return default_address
 
-    def contact_mechanism_get(self, types=None, usage=None):
-        """
-        Try to find a contact mechanism for the given types and usage, if no
-        usage matches the first mechanism of the given types is returned.
-        """
-        default_mechanism = None
-        if types:
-            if isinstance(types, str):
-                types = {types}
-            mechanisms = [m for m in self.contact_mechanisms
-                if m.type in types]
-        else:
-            mechanisms = self.contact_mechanisms
-        if mechanisms:
-            default_mechanism = mechanisms[0]
-            if usage:
-                for mechanism in mechanisms:
-                    if getattr(mechanism, usage):
-                        return mechanism
-        return default_mechanism
-
 
 class PartyLang(ModelSQL, ValueMixin):
     "Party Lang"
     __name__ = 'party.party.lang'
     party = fields.Many2One(
-        'party.party', "Party", ondelete='CASCADE', select=True)
+        'party.party', "Party", ondelete='CASCADE')
     lang = fields.Many2One('ir.lang', "Language")
 
     @classmethod
@@ -387,10 +376,10 @@ class PartyCategory(ModelSQL):
     'Party - Category'
     __name__ = 'party.party-party.category'
     _table = 'party_category_rel'
-    party = fields.Many2One('party.party', 'Party', ondelete='CASCADE',
-            required=True, select=True)
-    category = fields.Many2One('party.category', 'Category',
-        ondelete='CASCADE', required=True, select=True)
+    party = fields.Many2One(
+        'party.party', "Party", ondelete='CASCADE', required=True)
+    category = fields.Many2One(
+        'party.category', "Category", ondelete='CASCADE', required=True)
 
 
 IDENTIFIER_TYPES = [
@@ -613,11 +602,11 @@ class Identifier(sequence_ordered(), DeactivableMixin, ModelSQL, ModelView):
     'Party Identifier'
     __name__ = 'party.identifier'
     _rec_name = 'code'
-    party = fields.Many2One('party.party', 'Party', ondelete='CASCADE',
-        required=True, select=True,
+    party = fields.Many2One(
+        'party.party', "Party", ondelete='CASCADE', required=True,
         help="The party identified by this record.")
     address = fields.Many2One(
-        'party.address', "Address", ondelete='CASCADE', select=True,
+        'party.address', "Address", ondelete='CASCADE',
         states={
             'required': Eval('type_address', False),
             'invisible': ~Eval('type_address', True),
@@ -681,6 +670,11 @@ class Identifier(sequence_ordered(), DeactivableMixin, ModelSQL, ModelView):
     def _type_addresses(cls):
         return {'fr_siret'}
 
+    @fields.depends('address', '_parent_address.party')
+    def on_change_address(self):
+        if self.address:
+            self.party = self.address.party
+
     @fields.depends('type')
     def on_change_with_type_address(self, name=None):
         return self.type in self._type_addresses()
@@ -970,17 +964,18 @@ class Erase(Wizard):
                                 where=table.id.in_(query)))
                 if resource:
                     for Resource in resources:
-                        model_tables = [Resource.__table__()]
+                        model_tables = [
+                            (Resource.__table__(), Resource.resource)]
                         if Resource._history:
-                            model_tables.append(Resource.__table_history__())
-                        for table in model_tables:
+                            model_tables.append(
+                                (Resource.__table_history__(),
+                                    Resource.resource))
+                        for (table, resource_field) in model_tables:
                             cursor.execute(*table.delete(
                                     where=table.resource.like(
                                         Model.__name__ + ',%')
-                                    & Model.id.sql_cast(
-                                        Substring(table.resource,
-                                            Position(',', table.resource)
-                                            + Literal(1))).in_(query)))
+                                    & resource_field.sql_id(
+                                        table.resource, Model).in_(query)))
         return 'end'
 
     def check_erase(self, party):
